from abc import abstractmethod, ABC
from typing import Union

from . import GridLayer, CharactersLayer, TextLayer, UILayer, Layer


class LayerGroup(ABC):

    # Create layers

    @abstractmethod
    def createGridLayer(self, level: int = -1) -> GridLayer:
        raise NotImplementedError()

    @abstractmethod
    def createCharactersLayer(self, level: int = -1) -> CharactersLayer:
        raise NotImplementedError()

    @abstractmethod
    def createTextLayer(self, level: int = -1) -> TextLayer:
        raise NotImplementedError()

    @abstractmethod
    def createUILayer(self, level: int = -1) -> UILayer:
        raise NotImplementedError()

    # Get & set layers

    @abstractmethod
    def getLevelCount(self) -> int:
        raise NotImplementedError()

    @abstractmethod
    def getLayer(self, level: int) -> Layer:
        raise NotImplementedError()

    @abstractmethod
    def moveLayer(self, layer: Union[int, Layer], level: int):
        raise NotImplementedError()

    @abstractmethod
    def removeLayer(self, layer: Union[int, Layer]):
        raise NotImplementedError()

    # Properties

    @abstractmethod
    def setTranslation(self, x: float, y: float):
        raise NotImplementedError()

    # Commodity functions

    def setTileset(self, fileName: str):
        levelCount = self.getLevelCount()
        for level in range(levelCount):
            layer = self.getLayer(level)
            if layer is None:
                continue
            layer.setTileset(fileName)

    def setTileSize(self, tileWidth: int, tileHeight: int):
        levelCount = self.getLevelCount()
        for level in range(levelCount):
            layer = self.getLayer(level)
            if layer is None:
                continue
            layer.setTileSize(tileWidth, tileHeight)

    def draw(self):
        levelCount = self.getLevelCount()
        for level in range(levelCount):
            layer = self.getLayer(level)
            if layer is None:
                continue
            layer.draw()

    def deleteObjects(self):
        levelCount = self.getLevelCount()
        for level in range(levelCount):
            layer = self.getLayer(level)
            if layer is None:
                continue
            layer.deleteObjects()
