import logging

from gui import GUIFacadeFactory
from gui.input import KeyEvent, Keyboard
from mode import EditGameMode

logging.basicConfig(level=logging.INFO, format='\r%(filename)s:%(lineno)d: %(message)s')

# Create window
gui = GUIFacadeFactory().createInstance("OpenGL")
gui.createWindow(
    "OpenGL 2D Facade - https://www.patternsgameprog.com/",
    1280, 768
)
gui.init()

# Create a play game mode
mainMode = EditGameMode(gui)

# Run game
mainMode.init()
while not gui.closingRequested:

    # Handle inputs
    gui.updateInputs()
    if not mainMode.handleInputs():
        # If the mode didn't handle inputs, run a default handling
        keyboard = gui.keyboard
        for keyEvent in keyboard.keyEvents:
            if keyEvent.type == KeyEvent.KEYDOWN:
                if keyEvent.key == Keyboard.K_ESCAPE:
                    gui.closingRequested = True
                    break
        keyboard.clearKeyEvents()

    # Update game state
    mainMode.updateState()

    # Render scene
    gui.render()

# Delete objects
gui.quit()



