from __future__ import annotations
from typing import Dict, List

from .StateListener import StateListener
from .Region import Region


class State:

    def __init__(self):

        self.__listeners = []  # type: List[StateListener]
        self.__regions = {}  # type: Dict[str, Region]

    # Listeners

    def addListener(self, listener: StateListener):
        if listener not in self.__listeners:
            self.__listeners.append(listener)

    def removeListener(self, listener: StateListener):
        if listener in self.__listeners:
            self.__listeners.remove(listener)

    def notifyCurrentRegionChanged(self, regionName: str):
        for listener in self.__listeners:
            listener.currentRegionChanged(self, regionName)

    def notifyRegionCellChanged(self, regionName: str, x: int, y: int):
        for listener in self.__listeners:
            listener.regionCellChanged(self, regionName, x, y)

    # Data

    def createRegion(self, name: str, width: int, height: int):
        if name in self.__regions:
            raise ValueError("Region {} already exists".format(name))
        self.__regions[name] = Region(width, height)

    def getRegion(self, name: str) -> Region:
        if name not in self.__regions:
            raise ValueError("There is no region {}".format(name))
        return self.__regions[name]

